import java.awt.Color;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import java.util.*;
import java.io.*;

public class ImageManipulator 
{
	private static final int ALIGN_LEFT = 0;
	private static final int ALIGN_CENTER = 1;
	private static final int ALIGN_RIGHT = 2;
	
	private static boolean useAnsiEncoding = false; 
	
	public ImageManipulator() {
		useAnsiEncoding = true;
	}
	
	private static void drawString( Graphics g, String utf8String, int alignment, int posX, int posY ) {
		
		int xPos = posX;
		int yPos = posY;
		String ansiString;
		
		if ( useAnsiEncoding ) {
			try {
				ansiString = new String( utf8String.getBytes(), "UTF-8" );
			} catch ( UnsupportedEncodingException e ) {
				System.err.println( "INTERNAL ERROR: Character decoder not found, using unchanged message test." );
				ansiString = utf8String;
			}
		} else {
			ansiString = utf8String;
		}

		if ( alignment == ALIGN_CENTER || alignment == ALIGN_RIGHT) {
			// calculate the effective text len

			FontMetrics fm = g.getFontMetrics();
			int textLen = fm.stringWidth( ansiString );
			
			if ( alignment == ALIGN_CENTER ) {
				xPos = posX - textLen / 2;
			} else {
				// alignment == ALIGN_RIGHT
				xPos = posX - textLen;
			}
		}
		
		g.drawString( ansiString, xPos, yPos );
	}

	private static void writeStringUniversal(
		String sourceFileName, 
		String destFileName, 
		String message,
		Font aFont,
		Color aFontColor,
		int xPos,
		int yPos,
		int alignment) {

		BufferedImage img;
		try {
			img = ImageIO.read( new File ( sourceFileName ) );
		} catch ( IOException e ) {
			System.err.println( "Source file " + sourceFileName + " not read." );
			return;
		}
		
		Graphics g = img.getGraphics();
		g.setColor( aFontColor );
		g.setFont( aFont );
		
		drawString( g, message, alignment, xPos, yPos );
		
		int dotPos = destFileName.lastIndexOf( "." );
		if ( dotPos == 0 ) {
			System.err.println( "Destination file extension could not be determined." );
			return;
		}

		String ext = destFileName.substring( dotPos+1 );
		try {
			ImageIO.write( img, ext, new File ( destFileName ) );
		} catch ( IOException e ) {
			System.err.println( "Could not write image with type " + ext + " into file " + destFileName + "." );
			return;
		}
	}
	
	public static void writeStringLeftAdjust( 
		String sourceFileName, 
		String destFileName, 
		String message, 
		String fontName, 
		String fontAttributes, 
		String fontColor,
		Double dFontSize,
		Double dXPos,
		Double dYPos) {
		
		int fontSize 	= dFontSize.intValue();
		int xPos 		= dXPos.intValue();
		int yPos 		= dYPos.intValue();

		if ( xPos < 0 ) {
			xPos = 0;
		}
		if ( yPos < 0 ) {
			yPos = 0;
		}
		
		Color selectedFontColor = getColor( fontColor );
		if ( selectedFontColor == null ) {
			System.err.println( "Font color " + fontColor + " not found." );
			return;
		}
		
		Font myFont = getFont( fontName, fontAttributes, fontSize );
		if ( myFont == null ) {
			System.err.println( "Font " + fontName + " with attributes " + fontAttributes + " not found." );
			return;
		}
		
		writeStringUniversal( sourceFileName, destFileName, message, myFont, selectedFontColor, xPos, yPos, ALIGN_LEFT ); 
	}
	
	public static void writeStringRightAdjust( 
		String sourceFileName, 
		String destFileName, 
		String message, 
		String fontName, 
		String fontAttributes, 
		String fontColor,
		Double dFontSize,
		Double dXPos,
		Double dYPos) {
		
		int fontSize 	= dFontSize.intValue();
		int xPos 		= dXPos.intValue();
		int yPos 		= dYPos.intValue();

		if ( xPos < 0 ) {
			xPos = 0;
		}
		if ( yPos < 0 ) {
			yPos = 0;
		}
		
		Color selectedFontColor = getColor( fontColor );
		if ( selectedFontColor == null ) {
			System.err.println( "Font color " + fontColor + " not found." );
			return;
		}
		
		Font myFont = getFont( fontName, fontAttributes, fontSize );
		if ( myFont == null ) {
			System.err.println( "Font " + fontName + " with attributes " + fontAttributes + " not found." );
			return;
		}
		
		writeStringUniversal( sourceFileName, destFileName, message, myFont, selectedFontColor, xPos, yPos, ALIGN_RIGHT ); 
	}
	
	public static void writeStringCenterAdjust( 
		String sourceFileName, 
		String destFileName, 
		String message, 
		String fontName, 
		String fontAttributes, 
		String fontColor,
		Double dFontSize,
		Double dXPos,
		Double dYPos) {
		
		int fontSize 	= dFontSize.intValue();
		int xPos 		= dXPos.intValue();
		int yPos 		= dYPos.intValue();

		if ( xPos < 0 ) {
			xPos = 0;
		}
		if ( yPos < 0 ) {
			yPos = 0;
		}
		
		Color selectedFontColor = getColor( fontColor );
		if ( selectedFontColor == null ) {
			System.err.println( "Font color " + fontColor + " not found." );
			return;
		}
		
		Font myFont = getFont( fontName, fontAttributes, fontSize );
		if ( myFont == null ) {
			System.err.println( "Font " + fontName + " with attributes " + fontAttributes + " not found." );
			return;
		}
		
		writeStringUniversal( sourceFileName, destFileName, message, myFont, selectedFontColor, xPos, yPos, ALIGN_CENTER ); 
	}

	private static Font getFont( String aFontName, String someFontAttributes, int aFontSize ) {
		Font myFont;
		int attribs;
		
		attribs = Font.PLAIN;
		if ( someFontAttributes.equalsIgnoreCase( "BOLD" ) ) {
			attribs = Font.BOLD;
		}
		if ( someFontAttributes.equalsIgnoreCase( "ITALIC" ) ) {
			attribs = Font.ITALIC;
		}
		if ( someFontAttributes.equalsIgnoreCase( "BOLDITALIC" ) ) {
			attribs = Font.BOLD | Font.ITALIC;
		}
		
		myFont = new Font( aFontName, attribs, aFontSize );
		return myFont;
	}
	
	private static Color getColor( String colorName ) {
		Color myColor = null;
		
		if ( colorName.equalsIgnoreCase( "GREEN" ) ) {
			myColor = Color.GREEN;
		}
		if ( colorName.equalsIgnoreCase( "BLUE" ) ) {
			myColor = Color.BLUE;
		}
		if ( colorName.equalsIgnoreCase( "RED" ) ) {
			myColor = Color.RED;
		}
		if ( colorName.equalsIgnoreCase( "WHITE" ) ) {
			myColor = Color.WHITE;
		}
		if ( colorName.equalsIgnoreCase( "BLACK" ) ) {
			myColor = Color.BLACK;
		}
		if ( colorName.equalsIgnoreCase( "CYAN" ) ) {
			myColor = Color.CYAN;
		}
		if ( colorName.equalsIgnoreCase( "DARKGRAY" ) ) {
			myColor = Color.DARK_GRAY;
		}
		if ( colorName.equalsIgnoreCase( "GRAY" ) ) {
			myColor = Color.GRAY;
		}
		if ( colorName.equalsIgnoreCase( "LIGHTGRAY" ) ) {
			myColor = Color.LIGHT_GRAY;
		}
		if ( colorName.equalsIgnoreCase( "MAGENTA" ) ) {
			myColor = Color.MAGENTA;
		}
		if ( colorName.equalsIgnoreCase( "ORANGE" ) ) {
			myColor = Color.ORANGE;
		}
		if ( colorName.equalsIgnoreCase( "PINK" ) ) {
			myColor = Color.PINK;
		}
		if ( colorName.equalsIgnoreCase( "YELLOW" ) ) {
			myColor = Color.YELLOW;
		}
		if ( myColor == null ) {
			// try to get RGB value after the sign character
			// format is '#rrggbb', each color is represented in two characters
			if ( colorName.startsWith( "#" ) ) {
				String hexVal = colorName.substring( 1 );
				try {
					int r = Integer.parseInt( hexVal.substring( 0, 2 ), 16 );
					int g = Integer.parseInt( hexVal.substring( 2, 4 ), 16 );
					int b = Integer.parseInt( hexVal.substring( 4, 6 ), 16 );
					myColor = new Color( r, g, b );
				} catch (NumberFormatException e) {
					System.err.println( "Color '" + colorName + "' in not in this format '#rrggbb' e.g. '#07ABFF', using BLACK as default." );
					myColor = Color.BLACK;
				}
			}
		}
		return myColor;
	}

	
	private void testCase1() {
		ImageManipulator.writeStringLeftAdjust( "xxx.png", "xx2.png", "Menüpunkt", "Arial", "BOLD", "GREEN", new Double(12), new Double(20), new Double(20) );
		ImageManipulator.writeStringLeftAdjust( "xx2.png", "xx3.png", "black", "Arial", "BOLDITALIC", "BLACK", new Double(12), new Double(20), new Double(30) );
		ImageManipulator.writeStringLeftAdjust( "xx3.png", "xx4.png", "blue", "Arial", "PLAIN", "BLUE", new Double(9), new Double(20), new Double(40) );
		ImageManipulator.writeStringLeftAdjust( "xx4.png", "xx5.png", "red", "Courier", "ITALIC", "RED", new Double(9), new Double(20), new Double(50) );
		ImageManipulator.writeStringLeftAdjust( "xx5.png", "xx6.png", "correct", "Courier", "PLAIN", "#00FFAA", new Double(9), new Double(20), new Double(60) );
		ImageManipulator.writeStringLeftAdjust( "xx6.png", "xx7.png", "incorrect", "Courier", "PLAIN", "#00rFAA", new Double(9), new Double(20), new Double(70) );
		ImageManipulator.writeStringLeftAdjust( "xx7.png", "xx8.png", "äöüÄÖÜß§$&XYZ", "Courier", "PLAIN", "BLACK", new Double(9), new Double(20), new Double(80) );
	}
	
	private void testCase2() {
		ImageManipulator.writeStringRightAdjust( "xxx.png", "yx2.png", "Menüpunkt", "Arial", "BOLD", "GREEN", new Double(12), new Double(80), new Double(20) );
		ImageManipulator.writeStringRightAdjust( "yx2.png", "yx3.png", "black", "Arial", "BOLDITALIC", "BLACK", new Double(12), new Double(80), new Double(30) );
		ImageManipulator.writeStringRightAdjust( "yx3.png", "yx4.png", "blue", "Arial", "PLAIN", "BLUE", new Double(9), new Double(80), new Double(40) );
		ImageManipulator.writeStringRightAdjust( "yx4.png", "yx5.png", "red", "Courier", "ITALIC", "RED", new Double(9), new Double(80), new Double(50) );
		ImageManipulator.writeStringRightAdjust( "yx5.png", "yx6.png", "correct", "Courier", "PLAIN", "#00FFAA", new Double(9), new Double(80), new Double(60) );
		ImageManipulator.writeStringRightAdjust( "yx6.png", "yx7.png", "incorrect", "Courier", "PLAIN", "#00rFAA", new Double(9), new Double(80), new Double(70) );
		ImageManipulator.writeStringRightAdjust( "yx7.png", "yx8.png", "äöüÄÖÜß§$&XYZ", "Courier", "PLAIN", "BLACK", new Double(9), new Double(80), new Double(80) );
	}

	private void testCase3() {
		ImageManipulator.writeStringCenterAdjust( "xxx.png", "zx2.png", "Menüpunkt", "Arial", "BOLD", "GREEN", new Double(12), new Double(50), new Double(20) );
		ImageManipulator.writeStringCenterAdjust( "zx2.png", "zx3.png", "black", "Arial", "BOLDITALIC", "BLACK", new Double(12), new Double(50), new Double(30) );
		ImageManipulator.writeStringCenterAdjust( "zx3.png", "zx4.png", "blue", "Arial", "PLAIN", "BLUE", new Double(9), new Double(50), new Double(40) );
		ImageManipulator.writeStringCenterAdjust( "zx4.png", "zx5.png", "red", "Courier", "ITALIC", "RED", new Double(9), new Double(50), new Double(50) );
		ImageManipulator.writeStringCenterAdjust( "zx5.png", "zx6.png", "correct", "Courier", "PLAIN", "#00FFAA", new Double(9), new Double(50), new Double(60) );
		ImageManipulator.writeStringCenterAdjust( "zx6.png", "zx7.png", "incorrect", "Courier", "PLAIN", "#00rFAA", new Double(9), new Double(50), new Double(70) );
		ImageManipulator.writeStringCenterAdjust( "zx7.png", "zx8.png", "äöüÄÖÜß§$&XYZ", "Courier", "PLAIN", "BLACK", new Double(9), new Double(50), new Double(80) );
	}

	private void testCase4() {
		ImageManipulator.writeStringCenterAdjust( "xxx.png", "za1.png", "Copy of the original", "Arial", "PLAIN", "GREEN", new Double(10), new Double(50), new Double(20) );
		ImageManipulator.writeStringCenterAdjust( "za1.png", "za1.png", "Writing in the same file", "Arial", "PLAIN", "GREEN", new Double(10), new Double(50), new Double(60) );
	}

	public static void main(String[] args)
	{
		ImageManipulator im = new ImageManipulator();
		im.testCase1();
		im.testCase2();
		im.testCase3();
		im.testCase4();
	}


}
